==> typescript-angular-components-3.9.1.tgz: package/source/behaviors/autosave/autosave.ts <==
import { Directive, Input, Self, AfterViewInit, HostListener } from '@angular/core';
import { Observable, Subject } from 'rxjs';

import { FormComponent } from '../../components/form/form';
import { AutosaveActionService } from '../../services/autosaveAction/autosaveAction.service';
import { IWaitValue } from '../../services/async/async.service';

export const DEFAULT_AUTOSAVE_DEBOUNCE: number = 3000;

@Directive({
	selector: '[rlAutosave]',
})
export class AutosaveDirective implements AfterViewInit {
	@Input() saveWhenInvalid: boolean;
	@HostListener('keyup') keyupListener = this.resetDebounce;

	form: FormComponent;
	autosaveAction: AutosaveActionService;

	autosaveStart$: Subject<void> = new Subject<void>();
	autosaveCancel$: Subject<void> = new Subject<void>();

	constructor( @Self() form: FormComponent
			, autosaveAction: AutosaveActionService) {
		this.form = form;
		this.autosaveAction = autosaveAction;
	}

	ngAfterViewInit(): void {
		this.form.form.statusChanges.subscribe(this.setDebounce);
	}

	ngOnDestroy(): void {
		this.autosaveCancel$.next();
	}

	setDebounce = (): void => {
		if (this.canAutosave()) {
			this.autosaveCancel$.next();
			this.autosaveStart$.debounceTime(DEFAULT_AUTOSAVE_DEBOUNCE).takeUntil(this.autosaveCancel$).subscribe(() => this.autosave());
			this.autosaveStart$.next();
		} else {
			this.autosaveCancel$.next();
		}
	}

	resetDebounce(): void {
		if (this.canAutosave()) {
			this.autosaveStart$.next();
		}
	}

	autosave = (): void => {
		if (!this.canAutosave()) {
			return;
		}

		const waitOn = this.submitAndWait();
		if (waitOn) {
			// subscribes to kick off the stream
			this.autosaveAction.waitOn(waitOn).subscribe();
		}
	}

	submitAndWait(): IWaitValue<any> {
		if (this.saveWhenInvalid) {
			return this.form.saveForm();
		} else {
			return this.form.submitAndWait();
		}
	}

	private canAutosave(): boolean {
		return this.form.dirty && (this.saveWhenInvalid || this.form.validate());
	}
}

==> typescript-angular-components-4.2.8.tgz: package/source/behaviors/autosave/autosave.ts <==
import { Directive, Input, Self, AfterViewInit, HostListener } from '@angular/core';

import { FormComponent } from '../../components/form/form';
import { AutosaveActionService } from '../../services/autosaveAction/autosaveAction.service';
import { IWaitValue } from '../../services/async/async.service';

export const DEFAULT_AUTOSAVE_DEBOUNCE: number = 3000;

@Directive({
	selector: '[rlAutosave]',
})
export class AutosaveDirective implements AfterViewInit {
	@Input() saveWhenInvalid: boolean;
	@HostListener('keyup') keyupListener = this.resetDebounce;

	timer: any;
	form: FormComponent;
	autosaveAction: AutosaveActionService;

	constructor( @Self() form: FormComponent
			, autosaveAction: AutosaveActionService) {
		this.form = form;
		this.autosaveAction = autosaveAction;
	}

	ngAfterViewInit(): void {
		this.form.form.statusChanges.subscribe(this.setDebounce);
	}

	ngOnDestroy(): void {
		if (this.timer) {
			clearTimeout(this.timer);
		}
	}

	setDebounce = (): void => {
		if (this.canAutosave()) {
			if (!this.timer && this.form.dirty && (this.saveWhenInvalid || this.form.validate())) {
				this.timer = setTimeout(this.autosave, DEFAULT_AUTOSAVE_DEBOUNCE)
			}
		}
	}

	resetDebounce(): void {
		if (this.timer && this.canAutosave()) {
			clearTimeout(this.timer);
			this.timer = null;
			this.setDebounce();
		}
	}

	autosave = (): void => {
		if (this.canAutosave()) {
			const waitOn = this.submitAndWait();
			if (waitOn) {
				this.autosaveAction.waitOn(waitOn).subscribe();
			}
			clearTimeout(this.timer);
		}
	}

	submitAndWait(): IWaitValue<any> {
		if (this.saveWhenInvalid) {
			return this.form.saveForm();
		} else {
			return this.form.submitAndWait();
		}
	}

	private canAutosave(): boolean {
 		return this.form.dirty && (this.saveWhenInvalid || this.form.validate());
 	}
}
